from ocp_resources.resource import Resource


class SecurityContextConstraints(Resource):
    """
    Security Context Constraints object.
    """

    api_group = Resource.ApiGroup.SECURITY_OPENSHIFT_IO
